from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe


class AdminImagePreviewWidget(AdminFileWidget):
    """
    An admin widget that shows a preview of currently selected image.
    """

    def __init__(self, attrs=None, storage=None):
        super(AdminImagePreviewWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None, **kwargs):
        content = super(AdminImagePreviewWidget, self).render(
            name, value, attrs, **kwargs)
        return mark_safe(content + self._get_preview_tag(value))

    def _get_preview_tag(self, value):
        if value and hasattr(value, "url"):
            return '<p class="file-upload current-file-preview">Current Preview:<br />' \
                   '<img src="{0}" style="max-width: 300px;" /></p>'.format(value.url)
        return ''



import os
import shutil
from os.path import join, dirname

from django import test
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.management import call_command
from django.conf import settings

from image_helper.tests.test_app.models import TestModel
from image_helper.fields import _get_thumbnail_filename


class SizedImageFieldTests(test.TestCase):
    def setUp(self):
        call_command("migrate", verbosity=0)

    def tearDown(self):
        test_images_path = os.path.join(settings.MEDIA_ROOT, "test_images")
        if os.path.exists(test_images_path):
            shutil.rmtree(test_images_path)

    def _get_simple_uploaded_file(self):
        return SimpleUploadedFile(
            "sample_photo.png",
            open(join(dirname(__file__), "test_app/sample_photo.png"),
                 'rb').read(),
            content_type="image/png")

    def _save_image_model(self):
        upload_file = self._get_simple_uploaded_file()
        m = TestModel(image=upload_file)
        m.save()
        return m

    def test_saves_image_and_thumbnail(self):
        upload_file = self._get_simple_uploaded_file()
        m = TestModel(image=upload_file)
        m.save()

        model = TestModel.objects.get(pk=1)
        self.assertEqual("test_images/sample_photo.png", model.image.name)
        self.assertEqual("{}test_images/sample_photo.png".format(
            settings.MEDIA_URL), model.image.url)
        self.assertEqual("{}/test_images/sample_photo.png".format(
            settings.MEDIA_ROOT), model.image.path)
        self.assertEqual(True, os.path.exists(model.image.path))

        self.assertEqual("test_images/sample_photo-thumbnail.png",
                         model.image.thumbnail.name)
        self.assertEqual("{}test_images/sample_photo-thumbnail.png".format(
            settings.MEDIA_URL), model.image.thumbnail.url)
        self.assertEqual("{}/test_images/sample_photo-thumbnail.png".format(
            settings.MEDIA_ROOT), model.image.thumbnail.path)
        self.assertEqual(True, os.path.exists(model.image.thumbnail.path))

    def test_saves_appropriately_when_finding_valid_image_name(self):
        # Thumbnail should be named the same as image, just with the -thumbnail piece.
        self._save_image_model()
        self._save_image_model()

        model = TestModel.objects.get(pk=2)
        self.assertTrue(
            model.image.name.startswith("test_images/sample_photo"))
        self.assertNotEqual("test_images/sample_photo.png", model.image.name)
        self.assertEqual("{}{}".format(settings.MEDIA_URL, model.image.name),
                         model.image.url)
        self.assertEqual("{}/{}".format(settings.MEDIA_ROOT, model.image.name),
                         model.image.path)
        self.assertEqual(True, os.path.exists(model.image.path))

        expected_thumbnail_name = _get_thumbnail_filename(model.image.name)
        self.assertEqual(expected_thumbnail_name, model.image.thumbnail.name)
        self.assertEqual("{}{}".format(settings.MEDIA_URL,
                                       expected_thumbnail_name),
                         model.image.thumbnail.url)
        self.assertEqual("{}/{}".format(settings.MEDIA_ROOT,
                                        expected_thumbnail_name),
                         model.image.thumbnail.path)
        self.assertEqual(True, os.path.exists(model.image.thumbnail.path))


class GetThumbnailFilenameTests(test.TestCase):
    def test_get_thumbnail_filename(self):
        thumbnail_name = _get_thumbnail_filename("my_image.jpg")
        self.assertEqual("my_image-thumbnail.jpg", thumbnail_name)

    def test_allows_customized_append_text(self):
        thumbnail_name = _get_thumbnail_filename(
            "my_image.jpg", append_text="-small")
        self.assertEqual("my_image-small.jpg", thumbnail_name)



from django.contrib import admin
from django.utils.safestring import mark_safe

from image_helper.fields import SizedImageField
from image_helper.widgets import AdminImagePreviewWidget

from sample.models import TestModel


class TestModelAdmin(admin.ModelAdmin):
    list_display = ['preview_thumbnail', 'name']

    def preview_thumbnail(self, obj):
        return mark_safe('<img src="{}" />'.format(obj.image.thumbnail.url))

    formfield_overrides = {
        SizedImageField: {
            'widget': AdminImagePreviewWidget
        },
    }


admin.site.register(TestModel, TestModelAdmin)



from django.db import models

from image_helper.fields import SizedImageField


# Create your models here.
class TestModel(models.Model):
    name = models.CharField(max_length=20)
    image = SizedImageField(upload_to='sample_images', size=(220, 150), thumbnail_size=(100, 100))



from django.db import models
from image_helper.fields import SizedImageField


class TestModel(models.Model):
    image = SizedImageField(
        upload_to='test_images', size=(220, 150), thumbnail_size=(100, 100))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-07 17:18
from __future__ import unicode_literals

from django.db import migrations, models
import image_helper.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=20)),
                ('image', image_helper.fields.SizedImageField(upload_to='sample_images')),
            ],
        ),
    ]



import os

from io import BytesIO

import mimetypes

from django.db.models.fields.files import ImageField
from django.core.files.uploadedfile import SimpleUploadedFile
from django.db.models import signals

from PIL import Image

# todo: Add 'delete_with_model' option that will delete thumbnail and image when model is deleted.


def _get_thumbnail_filename(filename, append_text="-thumbnail"):
    """
    Returns a thumbnail version of the file name.
    """
    name, ext = os.path.splitext(filename)
    return ''.join([name, append_text, ext])


class ThumbnailField(object):
    """
    Instances of this class will be used to access data of the
    generated thumbnails. A thumbnail is created when the image is saved
    initially, but there's nothing persisted that references the thumbnail.
    When the `SizedImageField` is instantiated, it gets this thumbnail
    field attached to it where the thumbnail becomes accessible.

    for example: `image.thumbnail.url`
    """

    def __init__(self, name, storage):
        """
        Uses same storage as the parent field
        """
        self.name = name
        self.storage = storage

    @property
    def path(self):
        return self.storage.path(self.name)

    @property
    def url(self):
        return self.storage.url(self.name)

    @property
    def size(self):
        return self.storage.size(self.name)


class SizedImageField(ImageField):
    """
    An Image field that allows auto resizing auto creation of thumbnails.
    """

    def __init__(self,
                 verbose_name=None,
                 name=None,
                 width_field=None,
                 height_field=None,
                 size=None,
                 thumbnail_size=None,
                 **kwargs):
        """
        Added fields:
            - size: a tuple containing width and height to resize image, and
                an optional boolean setting if is wanted forcing that size
                (None for not resizing).
            - thumbnail_size: a tuple with same values than `size' (None for
                not creating a thumbnail

        Example: (640, 480, True) -> Will resize image to a width of 640px and
            a height of 480px. File will be cut if necessary for forcing
            the image to have the desired size
        """
        self.size = self._get_resize_options(size)
        self.thumbnail_size = self._get_resize_options(thumbnail_size)

        super(SizedImageField, self).__init__(verbose_name, name, width_field,
                                              height_field, **kwargs)

    def _get_resize_options(self, dimensions):
        """
        :param dimensions:
            A tuple of (width, height, force_size).
            'force_size' can be left off and will default to False.
        """
        if dimensions and isinstance(dimensions, (tuple, list)):
            if len(dimensions) < 3:
                dimensions = tuple(dimensions) + (False, )
            return dimensions

    def contribute_to_class(self, cls, name):
        """
        Makes sure thumbnail gets set when image field initialized.
        """
        super(SizedImageField, self).contribute_to_class(cls, name)
        signals.post_init.connect(self._set_thumbnail, sender=cls)

    def pre_save(self, model_instance, add):
        """
        Resizes, commits image to storage, and returns field's value just before saving.
        """
        file = getattr(model_instance, self.attname)
        if file and not file._committed:
            file.name = self._clean_file_name(model_instance, file.name)
            file.file = self._resize_image(model_instance, file)
            file.save(file.name, file, save=False)
        return file

    def _clean_file_name(self, model_instance, filename):
        """
        We need to make sure we know the full file name before we save the thumbnail so
        we can be sure the name doesn't change on save.

        This method gets the available filename and returns just the file part.
        """
        available_name = self.storage.get_available_name(
            self.generate_filename(model_instance, filename))
        return os.path.basename(available_name)

    def _create_thumbnail(self, model_instance, thumbnail, image_name):
        """
        Resizes and saves the thumbnail image
        """
        thumbnail = self._do_resize(thumbnail, self.thumbnail_size)
        full_image_name = self.generate_filename(model_instance, image_name)
        thumbnail_filename = _get_thumbnail_filename(full_image_name)
        thumb = self._get_simple_uploaded_file(thumbnail, thumbnail_filename)
        self.storage.save(thumbnail_filename, thumb)

    def _resize_image(self, model_instance, image_field):
        """"""
        image_name = image_field.name

        image = Image.open(image_field.file)
        if image.mode not in ('L', 'RGB'):
            image = image.convert('RGB')

        if self.size:
            image = self._do_resize(image, self.size)

        if self.thumbnail_size:
            self._create_thumbnail(model_instance, image.copy(), image_name)

        return self._get_simple_uploaded_file(image, image_name)

    def _do_resize(self, img, dimensions):
        width, height, force_size = dimensions
        if force_size:
            img.resize((width, height), Image.ANTIALIAS)
        else:
            img.thumbnail((width, height), Image.ANTIALIAS)
        return img

    def _set_thumbnail(self, instance=None, **kwargs):
        """
        Sets a `thumbnail` attribute on the image field class.
        On thumbnail you can access name, url, path attributes
        """
        image_field = getattr(instance, self.name)
        if image_field:
            thumbnail_filename = _get_thumbnail_filename(image_field.name)

            thumbnail_field = ThumbnailField(thumbnail_filename, self.storage)
            setattr(image_field, 'thumbnail', thumbnail_field)

    def _get_simple_uploaded_file(self, image, file_name):
        """
        :param image:
            a python PIL ``Image`` instance.

        :param file_name:
            The file name of the image.

        :returns:
            A django ``SimpleUploadedFile`` instance ready to be saved.
        """
        extension = os.path.splitext(file_name)[1]

        mimetype, encoding = mimetypes.guess_type(file_name)
        content_type = mimetype or 'image/png'

        temp_handle = BytesIO()
        image.save(temp_handle, self._get_pil_format(extension))
        temp_handle.seek(0)  # rewind the file

        suf = SimpleUploadedFile(
            file_name,
            temp_handle.read(),
            content_type=content_type,
        )
        return suf

    def _get_pil_format(self, extension):
        """
        :param extension:
            The file name extension (.png, .jpg, etc...)

        :returns:
            The file format PIL needs from the file extension.
            Eg. PNG or JPEG
        """
        return Image.EXTENSION[extension.lower()]
